@@ -64,10 +64,9 @@ function init_common_tricircle_conf {
 function init_local_neutron_conf {
 
     iniset $NEUTRON_CONF DEFAULT core_plugin tricircle.network.local_plugin.TricirclePlugin
-    iniset $NEUTRON_CONF DEFAULT service_plugins tricircle.network.local_l3_plugin.TricircleL3Plugin
 
-    iniset $NEUTRON_CONF client auth_url http://$KEYSTONE_SERVICE_HOST:5000/v3
-    iniset $NEUTRON_CONF client identity_url http://$KEYSTONE_SERVICE_HOST:35357/v3
+    iniset $NEUTRON_CONF client auth_url http://$KEYSTONE_SERVICE_HOST/identity/v3
+    iniset $NEUTRON_CONF client identity_url http://$KEYSTONE_SERVICE_HOST/identity_admin/v3
     iniset $NEUTRON_CONF client admin_username admin
     iniset $NEUTRON_CONF client admin_password $ADMIN_PASSWORD
     iniset $NEUTRON_CONF client admin_tenant demo
@@ -96,6 +95,7 @@ function init_local_neutron_variables {
         local vlan_ranges=(network_vlan_ranges=$vlan_option,$ext_option)
         Q_ML2_PLUGIN_VLAN_TYPE_OPTIONS=$vlan_ranges
         Q_ML2_PLUGIN_VXLAN_TYPE_OPTIONS="vni_ranges=$TRICIRCLE_DEFAULT_VXLAN_RANGE"
+        Q_ML2_PLUGIN_FLAT_TYPE_OPTIONS="flat_networks=$TRICIRCLE_DEFAULT_FLAT_NETWORKS"
 
         local vlan_mapping="bridge:$TRICIRCLE_DEFAULT_VLAN_BRIDGE"
         local ext_mapping="extern:$TRICIRCLE_DEFAULT_EXT_BRIDGE"
@@ -161,6 +161,7 @@ function configure_tricircle_api_wsgi {
 
     sudo cp $TRICIRCLE_API_APACHE_TEMPLATE $tricircle_api_apache_conf
     sudo sed -e "
+        s|%TRICIRCLE_BIN%|$tricircle_bin_dir|g;
         s|%PUBLICPORT%|$TRICIRCLE_API_PORT|g;
         s|%APACHE_NAME%|$APACHE_NAME|g;
         s|%PUBLICWSGI%|$tricircle_bin_dir/tricircle-api-wsgi|g;
@@ -249,6 +250,11 @@ function start_central_neutron_server {
         tenant_network_types+=,vxlan
         iniset $NEUTRON_CONF.$server_index tricircle vni_ranges `echo $Q_ML2_PLUGIN_VXLAN_TYPE_OPTIONS | awk -F= '{print $2}'`
     fi
+    if [ "Q_ML2_PLUGIN_FLAT_TYPE_OPTIONS" != "" ]; then
+        type_drivers+=,flat
+        tenant_network_types+=,flat
+        iniset $NEUTRON_CONF.$server_index tricircle flat_networks `echo $Q_ML2_PLUGIN_FLAT_TYPE_OPTIONS | awk -F= '{print $2}'`
+    fi
     iniset $NEUTRON_CONF.$server_index tricircle type_drivers $type_drivers
     iniset $NEUTRON_CONF.$server_index tricircle tenant_network_types $tenant_network_types
     iniset $NEUTRON_CONF.$server_index tricircle enable_api_gateway False
