@@ -19,9 +19,8 @@ from oslo_log import log as logging
 
 from tricircle.common import constants
 import tricircle.common.context as t_context
-import tricircle.common.exceptions as t_exc
+import tricircle.common.exceptions as t_exceptions
 from tricircle.common.i18n import _
-from tricircle.common.i18n import _LE
 from tricircle.common import policy
 from tricircle.common import utils
 
@@ -81,8 +80,8 @@ class RoutingController(rest.RestController):
                 return utils.format_api_error(
                     409, _('Resource routing already exists'))
         except Exception as e:
-            LOG.exception(_LE('Failed to create resource routing: '
-                              '%(exception)s '), {'exception': e})
+            LOG.exception('Failed to create resource routing: '
+                          '%(exception)s ', {'exception': e})
             return utils.format_api_error(
                 500, _('Failed to create resource routing'))
 
@@ -120,8 +119,8 @@ class RoutingController(rest.RestController):
             return {'routings': db_api.list_resource_routings(context,
                                                               filters)}
         except Exception as e:
-            LOG.exception(_LE('Failed to show all resource routings: '
-                              '%(exception)s '), {'exception': e})
+            LOG.exception('Failed to show all resource routings: '
+                          '%(exception)s ', {'exception': e})
             return utils.format_api_error(
                 500, _('Failed to show all resource routings'))
 
@@ -135,7 +134,7 @@ class RoutingController(rest.RestController):
 
         try:
             return {'routing': db_api.get_resource_routing(context, _id)}
-        except t_exc.ResourceNotFound:
+        except t_exceptions.ResourceNotFound:
             return utils.format_api_error(
                 404, _('Resource routing not found'))
 
@@ -149,7 +148,7 @@ class RoutingController(rest.RestController):
 
         try:
             db_api.get_resource_routing(context, _id)
-        except t_exc.ResourceNotFound:
+        except t_exceptions.ResourceNotFound:
             return utils.format_api_error(404,
                                           _('Resource routing not found'))
         try:
@@ -157,8 +156,8 @@ class RoutingController(rest.RestController):
             pecan.response.status = 200
             return pecan.response
         except Exception as e:
-            LOG.exception(_LE('Failed to delete the resource routing: '
-                              '%(exception)s '), {'exception': e})
+            LOG.exception('Failed to delete the resource routing: '
+                          '%(exception)s ', {'exception': e})
             return utils.format_api_error(
                 500, _('Failed to delete the resource routing'))
 
@@ -172,7 +171,7 @@ class RoutingController(rest.RestController):
 
         try:
             db_api.get_resource_routing(context, _id)
-        except t_exc.ResourceNotFound:
+        except t_exceptions.ResourceNotFound:
             return utils.format_api_error(404,
                                           _('Resource routing not found'))
 
@@ -203,13 +202,13 @@ class RoutingController(rest.RestController):
             try:
                 # find the pod through the pod_id and verify whether it exists
                 db_api.get_pod(context, new_pod_id)
-            except t_exc.ResourceNotFound:
+            except t_exceptions.ResourceNotFound:
                 return utils.format_api_error(
                     400, _("The pod %(new_pod_id)s doesn't"
                            " exist") % {'new_pod_id': new_pod_id})
             except Exception as e:
-                LOG.exception(_LE('Failed to update resource routing: '
-                                  '%(exception)s '), {'exception': e})
+                LOG.exception('Failed to update resource routing: '
+                              '%(exception)s ', {'exception': e})
                 return utils.format_api_error(
                     500, _('Failed to update resource routing'))
 
@@ -218,7 +217,7 @@ class RoutingController(rest.RestController):
                 context, _id, update_dict)
             return {'routing': routing_updated}
         except Exception as e:
-            LOG.exception(_LE('Failed to update resource routing: '
-                              '%(exception)s '), {'exception': e})
+            LOG.exception('Failed to update resource routing: '
+                          '%(exception)s ', {'exception': e})
             return utils.format_api_error(
                 500, _('Failed to update resource routing'))
