@@ -28,7 +28,6 @@ from oslo_log import log as logging
 
 import tricircle.common.context as tricircle_context
 from tricircle.common import exceptions
-from tricircle.common.i18n import _LW
 from tricircle.common import resource_handle
 from tricircle.db import api
 from tricircle.db import models
@@ -36,10 +35,10 @@ from tricircle.db import models
 
 client_opts = [
     cfg.StrOpt('auth_url',
-               default='http://127.0.0.1:5000/v3',
+               default='http://127.0.0.1/identity/v3',
                help='keystone authorization url'),
     cfg.StrOpt('identity_url',
-               default='http://127.0.0.1:35357/v3',
+               default='http://127.0.0.1/identity_admin/v3',
                help='keystone service url'),
     cfg.BoolOpt('auto_refresh_endpoint',
                 default=False,
@@ -98,8 +97,8 @@ def _safe_operation(operation_name):
                     if i == retries:
                         raise
                     if cfg.CONF.client.auto_refresh_endpoint:
-                        LOG.warning(_LW('%(exception)s, '
-                                        'update endpoint and try again'),
+                        LOG.warning('%(exception)s, '
+                                    'update endpoint and try again',
                                     {'exception': e.message})
                         instance._update_endpoint_from_keystone(context, True)
                     else:
@@ -109,8 +108,8 @@ def _safe_operation(operation_name):
                     # for the given pod and service, we add default behaviours
                     # for the handle functions
                     if i < retries and cfg.CONF.client.auto_refresh_endpoint:
-                        LOG.warning(_LW('%(exception)s, '
-                                        'update endpoint and try again'),
+                        LOG.warning('%(exception)s, '
+                                    'update endpoint and try again',
                                     {'exception': e.message})
                         instance._update_endpoint_from_keystone(context, True)
                         continue
