@@ -15,6 +15,7 @@
 
 import collections
 import copy
+import re
 import six
 
 from oslo_config import cfg
@@ -26,8 +27,10 @@ from neutron.api.v2 import attributes
 from neutron.callbacks import events
 from neutron.callbacks import registry
 from neutron.callbacks import resources
+import neutron.common.exceptions as ml2_exceptions
+from neutron.db import _resource_extend as resource_extend
+from neutron.db import api as q_db_api
 from neutron.db.availability_zone import router as router_az
-from neutron.db import common_db_mixin
 from neutron.db import db_base_plugin_v2
 from neutron.db import external_net_db
 from neutron.db import extradhcpopt_db
@@ -57,7 +60,7 @@ import tricircle.common.client as t_client
 import tricircle.common.constants as t_constants
 import tricircle.common.context as t_context
 import tricircle.common.exceptions as t_exceptions
-from tricircle.common.i18n import _, _LE, _LI, _LW
+from tricircle.common.i18n import _
 from tricircle.common import xrpcapi
 import tricircle.db.api as db_api
 from tricircle.db import core
@@ -90,6 +93,12 @@ tricircle_opts = [
                 help=_('Comma-separated list of <vni_min>:<vni_max> tuples '
                        'enumerating ranges of VXLAN VNI IDs that are '
                        'available for tenant network allocation.')),
+    cfg.ListOpt('flat_networks',
+                default='*',
+                help=_("List of physical_network names with which flat "
+                       "networks can be created. Use default '*' to allow "
+                       "flat networks with arbitrary physical_network names. "
+                       "Use an empty list to disable flat networks.")),
     cfg.StrOpt('bridge_network_type',
                default='vxlan',
                help=_('Type of l3 bridge network, this type should be enabled '
@@ -151,7 +160,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
     def __init__(self):
         super(TricirclePlugin, self).__init__()
-        LOG.info(_LI("Starting Tricircle Neutron Plugin"))
+        LOG.info("Starting Tricircle Neutron Plugin")
         self.clients = {}
         self.xjob_handler = xrpcapi.XJobAPI()
         self._setup_rpc()
@@ -209,13 +218,11 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                     availability_zone=diff.pop())
 
     @staticmethod
+    @resource_extend.extends([attributes.NETWORKS])
     def _extend_availability_zone(net_res, net_db):
         net_res[az_ext.AZ_HINTS] = az_ext.convert_az_string_to_list(
             net_db[az_ext.AZ_HINTS])
 
-    common_db_mixin.CommonDbMixin.register_dict_extend_funcs(
-        attributes.NETWORKS, ['_extend_availability_zone'])
-
     @staticmethod
     def _ensure_az_set_for_external_network(context, req_data):
         external = req_data.get(external_net.EXTERNAL)
@@ -289,8 +296,11 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         if az_ext.AZ_HINTS in net_data:
             self._validate_availability_zones(context,
                                               net_data[az_ext.AZ_HINTS])
-        with context.session.begin(subtransactions=True):
-            res = super(TricirclePlugin, self).create_network(context, network)
+        with q_db_api.context_manager.writer.using(context):
+            net_db = self.create_network_db(context, network)
+            res = self._make_network_dict(net_db, process_extensions=False,
+                                          context=context)
+            self._process_l3_create(context, res, net_data)
             net_data['id'] = res['id']
             self.type_manager.create_network_segments(context, net_data,
                                                       tenant_id)
@@ -298,17 +308,23 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             if az_ext.AZ_HINTS in net_data:
                 az_hints = az_ext.convert_az_list_to_string(
                     net_data[az_ext.AZ_HINTS])
-                update_res = super(TricirclePlugin, self).update_network(
-                    context, res['id'],
-                    {'network': {az_ext.AZ_HINTS: az_hints}})
-                res[az_ext.AZ_HINTS] = update_res[az_ext.AZ_HINTS]
-            self._process_l3_create(context, res, net_data)
+                net_db[az_ext.AZ_HINTS] = az_hints
+                res[az_ext.AZ_HINTS] = net_data[az_ext.AZ_HINTS]
             # put inside a session so when bottom operations fails db can
             # rollback
             if is_external:
                 self._fill_provider_info(res, net_data)
-                self._create_bottom_external_network(
-                    context, net_data, res['id'])
+                try:
+                    self._create_bottom_external_network(
+                        context, net_data, res['id'])
+                except q_cli_exceptions.Conflict as e:
+                    pattern = re.compile('Physical network (.*) is in use')
+                    match = pattern.search(e.message)
+                    if not match:
+                        raise
+                    else:
+                        raise ml2_exceptions.FlatNetworkInUse(
+                            physical_network=match.groups()[0])
         return res
 
     def delete_network(self, context, network_id):
@@ -393,14 +409,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             net[az_ext.AZ_HINTS] = self._convert_az2region(t_ctx, az_hints)
 
     def _convert_az2region(self, t_ctx, az_hints):
-        region_names = set()
-        for az_hint in az_hints:
-            pods = db_api.find_pods_by_az_or_region(t_ctx, az_hint)
-            if not pods:
-                continue
-            for pod in pods:
-                region_names.add(pod['region_name'])
-        return list(region_names)
+        return self.helper.convert_az2region(t_ctx, az_hints)
 
     def get_network(self, context, network_id, fields=None):
         net = super(TricirclePlugin, self).get_network(context, network_id,
@@ -426,21 +435,30 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
     def create_subnet(self, context, subnet):
         subnet_data = subnet['subnet']
         network = self.get_network(context, subnet_data['network_id'])
+        is_external = network.get(external_net.EXTERNAL)
         with context.session.begin(subtransactions=True):
             res = super(TricirclePlugin, self).create_subnet(context, subnet)
             # put inside a session so when bottom operations fails db can
             # rollback
-            if network.get(external_net.EXTERNAL):
+            if is_external:
                 self._create_bottom_external_subnet(
                     context, res, network, res['id'])
-        if res['enable_dhcp']:
-            try:
-                t_ctx = t_context.get_context_from_neutron_context(context)
+        snat_port_id = None
+        try:
+            t_ctx = t_context.get_context_from_neutron_context(context)
+            if not subnet_data['name'].startswith(
+                    t_constants.bridge_subnet_name[:-3]) and not is_external:
+                # do not reserve snat port for bridge and external subnet
+                snat_port_id = self.helper.prepare_top_snat_port(
+                    t_ctx, context, res['tenant_id'], network['id'], res['id'])
+            if res['enable_dhcp']:
                 self.helper.prepare_top_dhcp_port(
                     t_ctx, context, res['tenant_id'], network['id'], res['id'])
-            except Exception:
-                self.delete_subnet(context, res['id'])
-                raise
+        except Exception:
+            if snat_port_id:
+                super(TricirclePlugin, self).delete_port(context, snat_port_id)
+            self.delete_subnet(context, res['id'])
+            raise
         return res
 
     def _delete_pre_created_port(self, t_ctx, q_ctx, port_name):
@@ -477,6 +495,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             raise
         dhcp_port_name = t_constants.dhcp_port_name % subnet_id
         self._delete_pre_created_port(t_ctx, context, dhcp_port_name)
+        snat_port_name = t_constants.snat_port_name % subnet_id
+        self._delete_pre_created_port(t_ctx, context, snat_port_name)
         super(TricirclePlugin, self).delete_subnet(context, subnet_id)
 
     def update_subnet(self, context, subnet_id, subnet):
@@ -568,7 +588,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                 updated_sens_attrs.append('%s = %s' % (key, request_body[key]))
 
         warning_attrs = ', '.join(updated_sens_attrs)
-        LOG.warning(_LW('update port: %(port_id)s , %(warning_attrs)s'),
+        LOG.warning('update port: %(port_id)s , %(warning_attrs)s',
                     {'port_id': port_id, 'warning_attrs': warning_attrs})
 
     def _handle_bottom_security_group(self, t_ctx, top_sg, bottom_pod):
@@ -617,9 +637,19 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                       'following routers: %s, xjob triggered',
                       port_body['network_id'], router_ids)
             admin_context = t_context.get_admin_context()
-            self.xjob_handler.setup_bottom_router(
-                admin_context, port_body['network_id'],
-                router_ids[0], pod['pod_id'])
+            t_ctx = t_context.get_context_from_neutron_context(context)
+
+            for router_id in router_ids:
+                router = self.get_router(context, router_id)
+                if not self.helper.is_local_router(t_ctx, router):
+                    # for local router, job will be triggered after router
+                    # interface attachment.
+                    self.xjob_handler.setup_bottom_router(
+                        admin_context, port_body['network_id'],
+                        router_id, pod['pod_id'])
+                    # network will be attached to only one non-local router,
+                    # so we break here
+                    break
         else:
             LOG.debug('Update port: no interfaces found, xjob not'
                       'triggered')
@@ -698,8 +728,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                         b_client.update_ports(t_ctx, b_port_id, port)
                     except q_cli_exceptions.NotFound:
                         LOG.error(
-                            _LE('port: %(port_id)s not found, '
-                                'region name: %(name)s'),
+                            ('port: %(port_id)s not found, '
+                             'region name: %(name)s'),
                             {'port_id': b_port_id, 'name': b_region_name})
 
                     if request_body.get('security_groups', None):
@@ -1327,6 +1357,15 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             net_id = subnet['network_id']
         return net_id
 
+    def _get_subnet_id_by_interface_info(self, q_ctx, add_by_port,
+                                         interface_info):
+        if add_by_port:
+            port = self.get_port(q_ctx, interface_info['port_id'])
+            # here we assume the port has an IP
+            return port['fixed_ips'][0]['subnet_id']
+        else:
+            return interface_info['subnet_id']
+
     def _get_net_pods_by_interface_info(self, t_ctx, q_ctx, add_by_port,
                                         interface_info):
         net_id = self._get_net_id_by_interface_info(q_ctx, add_by_port,
@@ -1519,6 +1558,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         return ret
 
     def validate_router_net_location_match(self, t_ctx, router, net):
+        is_local_router = self.helper.is_local_router(t_ctx, router)
         router_az_hints = self.helper.get_router_az_hints(router)
         if not router_az_hints:
             return
@@ -1527,6 +1567,12 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
         net_az_hints = net.get(az_ext.AZ_HINTS)
         if not net_az_hints:
+            if is_local_router:
+                # network az hints parameter is not specified, meaning that
+                # this network can be located in any pod, such network is
+                # allowed to be attached to a local router, for supporting
+                # multi-gateway l3 mode
+                return
             raise t_exceptions.RouterNetworkLocationMismatch(
                 router_az_hints=router_region_names,
                 net_az_hints=['All Region'])
@@ -1537,6 +1583,17 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         else:
             net_region_names = self._convert_az2region(t_ctx, net_az_hints)
         net_region_set = set(net_region_names)
+
+        if is_local_router:
+            if router_region_set <= net_region_set:
+                # pods that this network can be located include the pod of the
+                # local router, this attachment is allowed, for supporting
+                # multi-gateway l3 mode
+                return
+            raise t_exceptions.RouterNetworkLocationMismatch(
+                router_az_hints=router_region_names,
+                net_az_hints=net_region_names)
+
         diff = net_region_set - router_region_set
         if diff:
             raise t_exceptions.RouterNetworkLocationMismatch(
@@ -1551,10 +1608,34 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         add_by_port, _ = self._validate_interface_info(interface_info)
         net_id = self._get_net_id_by_interface_info(
             context, add_by_port, interface_info)
+        subnet_id = self._get_subnet_id_by_interface_info(
+            context, add_by_port, interface_info)
         net = self.get_network(context, net_id)
+        subnet = self.get_subnet(context, subnet_id)
         self.validate_router_net_location_match(t_ctx, router, net)
         is_local_router = self.helper.is_local_router(t_ctx, router)
 
+        if is_local_router:
+            other_infs = super(TricirclePlugin, self).get_ports(
+                context, filters={
+                    'network_id': [net_id],
+                    'device_owner': [constants.DEVICE_OWNER_ROUTER_INTF]})
+            for other_inf in other_infs:
+                if not other_inf['device_id']:
+                    continue
+                other_ip = other_inf['fixed_ips'][0]['ip_address']
+                other_router = super(
+                    TricirclePlugin, self).get_router(context,
+                                                      other_inf['device_id'])
+                if not self.helper.is_local_router(t_ctx, other_router) and (
+                        other_ip == subnet['gateway_ip']):
+                    # this network has already been attached to a non-local
+                    # router and the gateway port is on that router, in this
+                    # case, we don't allow this network to be attached other
+                    # local routers
+                    raise t_network_exc.NetAttachedToNonLocalRouter(
+                        network_id=net_id, router_id=other_inf['device_id'])
+
         t_pod = db_api.get_top_pod(t_ctx)
         assert t_pod
 
@@ -1566,8 +1647,24 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             self._get_bridge_network_subnet(
                 t_ctx, context, project_id, t_pod, pool_id)
 
+        if is_local_router:
+            if self.helper.is_local_network(t_ctx, net):
+                router_region = self.helper.get_router_az_hints(router)[0]
+                b_client = self._get_client(router_region)
+                b_pod = db_api.get_pod_by_name(t_ctx, router_region)
+                # get bottom network will create bottom network and subnet
+                b_client.get_networks(t_ctx, net_id)
+                # create resource mapping so job will be triggered
+                db_api.create_resource_mapping(
+                    t_ctx, net_id, net_id, b_pod['pod_id'], net['project_id'],
+                    t_constants.RT_NETWORK)
+                db_api.create_resource_mapping(
+                    t_ctx, subnet_id, subnet_id, b_pod['pod_id'],
+                    subnet['project_id'], t_constants.RT_SUBNET)
+
         return_info = super(TricirclePlugin, self).add_router_interface(
             context, router_id, interface_info)
+
         _, b_pods = self._get_net_pods_by_interface_info(
             t_ctx, context, add_by_port, interface_info)
         if not b_pods:
@@ -1681,8 +1778,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                     floating_ip.update(org_data)
         except Exception as e:
             # log the exception and re-raise it
-            LOG.exception(_LE('Fail to rollback floating ip data, reason: '
-                              '%(reason)s') % {'reason': e.message})
+            LOG.exception('Fail to rollback floating ip data, reason: '
+                          '%(reason)s' % {'reason': e.message})
             raise
 
     def update_floatingip(self, context, _id, floatingip):
@@ -1708,9 +1805,9 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             # NOTE(zhiyuan) when exception occurs, we update floating ip object
             # to rollback fixed_port_id, fixed_ip_address, router_id
             LOG.exception(
-                _LE('Fail to update floating ip, reason: '
-                    '%(reason)s, rollback floating ip data') % {
-                    'reason': e.message})
+                'Fail to update floating ip, reason: '
+                '%(reason)s, rollback floating ip data' %
+                {'reason': e.message})
             org_data = {
                 'fixed_port_id': org_floatingip_dict['port_id'],
                 'fixed_ip_address': org_floatingip_dict['fixed_ip_address'],
@@ -1754,8 +1851,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             # top and bottom internal port, this is an inconsistent state,
             # but since bottom internal port does not exist, no operation
             # in bottom pod is required
-            LOG.warning(_LW('Internal port associated with floating ip '
-                            'does not exist in bottom pod.'))
+            LOG.warning('Internal port associated with floating ip '
+                        'does not exist in bottom pod.')
             return
 
         b_int_net_pod, b_int_port_id = mappings[0]
