@@ -32,7 +32,7 @@ from neutron.plugins.ml2 import plugin
 from tricircle.common import client
 import tricircle.common.constants as t_constants
 import tricircle.common.context as t_context
-from tricircle.common.i18n import _, _LE
+from tricircle.common.i18n import _
 
 from tricircle.common import resource_handle
 import tricircle.common.utils as t_utils
@@ -286,9 +286,9 @@ class TricirclePlugin(plugin.Ml2Plugin):
                 located = self._is_network_located_in_region(network,
                                                              region_name)
                 if not located:
-                    LOG.error(_LE('network: %(net_id)s not located in current '
-                                  'region: %(region_name)s, '
-                                  'az_hints: %(az_hints)s'),
+                    LOG.error('network: %(net_id)s not located in current '
+                              'region: %(region_name)s, '
+                              'az_hints: %(az_hints)s',
                               {'net_id': network['id'],
                                'region_name': region_name,
                                'az_hints': network[az_ext.AZ_HINTS]})
@@ -402,7 +402,7 @@ class TricirclePlugin(plugin.Ml2Plugin):
         t_ctx = t_context.get_context_from_neutron_context(context)
         b_subnet = self.core_plugin.get_subnet(context, _id)
         origin_enable_dhcp = b_subnet['enable_dhcp']
-        req_enable_dhcp = subnet['subnet']['enable_dhcp']
+        req_enable_dhcp = subnet['subnet'].get('enable_dhcp')
         # when request enable dhcp, and origin dhcp is disabled,
         # ensure subnet dhcp port is created
         if req_enable_dhcp and not origin_enable_dhcp:
@@ -424,7 +424,15 @@ class TricirclePlugin(plugin.Ml2Plugin):
             return
         subnet_id = port['fixed_ips'][0]['subnet_id']
         t_subnet = self.neutron_handle.handle_get(t_ctx, 'subnet', subnet_id)
-        port['fixed_ips'][0]['ip_address'] = t_subnet['gateway_ip']
+        snat_port_name = t_constants.snat_port_name % t_subnet['id']
+        raw_client = self.neutron_handle._get_client(t_ctx)
+        params = {'name': snat_port_name}
+        t_ports = raw_client.list_ports(**params)['ports']
+        if not t_ports:
+            raise t_exceptions.CentralizedSNATPortNotFound(
+                subnet_id=t_subnet['id'])
+        port['fixed_ips'][0][
+            'ip_address'] = t_ports[0]['fixed_ips'][0]['ip_address']
 
     def create_port_bulk(self, context, ports):
         # NOTE(zhiyuan) currently this bulk operation is only for shadow port
@@ -457,7 +465,9 @@ class TricirclePlugin(plugin.Ml2Plugin):
                     ip_address=fixed_ip['ip_address'])
             return t_ports[0]
 
-        if port_body['fixed_ips'] is not q_constants.ATTR_NOT_SPECIFIED:
+        if port_body['fixed_ips'] is not q_constants.ATTR_NOT_SPECIFIED and (
+            port_body.get('device_owner') != (
+                q_constants.DEVICE_OWNER_LOADBALANCERV2)):
             if not self._is_special_port(port_body):
                 fixed_ip = port_body['fixed_ips'][0]
                 ip_address = fixed_ip.get('ip_address')
@@ -566,8 +576,8 @@ class TricirclePlugin(plugin.Ml2Plugin):
                                                  profile_dict)
         elif cfg.CONF.client.cross_pod_vxlan_mode == t_constants.NM_L2GW:
             if not cfg.CONF.tricircle.l2gw_tunnel_ip:
-                LOG.error(_LE('Cross-pod VxLAN networking mode is set to l2gw '
-                              'but L2 gateway tunnel ip is not configured'))
+                LOG.error('Cross-pod VxLAN networking mode is set to l2gw '
+                          'but L2 gateway tunnel ip is not configured')
                 return
             l2gw_tunnel_ip = cfg.CONF.tricircle.l2gw_tunnel_ip
             helper.NetworkHelper.fill_agent_data(agent_type, host, agents[0],
@@ -682,6 +692,8 @@ class TricirclePlugin(plugin.Ml2Plugin):
         self.core_plugin.delete_port(context, _id, l3_port_check)
 
     def _handle_security_group(self, t_ctx, q_ctx, port):
+        if 'security_groups' not in port:
+            return
         if not port['security_groups']:
             raw_client = self.neutron_handle._get_client(t_ctx)
             params = {'name': 'default'}
